/*-
 * Copyright (C) @BABOLO  2001 Jul 05
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * $Id: ra-filter.c,v 1.3 2002/09/26 06:16:22 babolo Exp $ 
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sysexits.h>
#include <sys/uio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <netdb.h>
#include <stdio.h>
#include <err.h>


#define argtimeval timeval                       /* borrowed from argus (Carnegie Mellon University) */
#define arg_uint8   u_char
#define arg_int8    char
#define arg_uint16  u_short
#define arg_int16   short
#define arg_uint32  u_int
#define arg_int32   int                      /* END borrowed from argus (Carnegie Mellon University) */

#include <cons_def.h>
#include <cons_out.h>
#include "bufread.h"
#include "babololib.h"

/* typedef u_int32_t in_addr_t; For old FreeBSD */
#define FTREE_STRING "ftree"

char *myname;
u_int32_t masks[] =
{ 0x00000000
, 0x00000080, 0x000000C0, 0x000000E0, 0x000000F0
, 0x000000F8, 0x000000FC, 0x000000FE, 0x000000FF
, 0x000080FF, 0x0000C0FF, 0x0000E0FF, 0x0000F0FF
, 0x0000F8FF, 0x0000FCFF, 0x0000FEFF, 0x0000FFFF
, 0x0080FFFF, 0x00C0FFFF, 0x00E0FFFF, 0x00F0FFFF
, 0x00F8FFFF, 0x00FCFFFF, 0x00FEFFFF, 0x00FFFFFF
, 0x80FFFFFF, 0xC0FFFFFF, 0xE0FFFFFF, 0xF0FFFFFF
, 0xF8FFFFFF, 0xFCFFFFFF, 0xFEFFFFFF, 0xFFFFFFFF
};

int main(int argc, char *const *argv)
{   struct hostent *hp;
    in_addr_t lookfor;
    int written, n;
    char *target;
    struct WriteStruct head, next;
    u_int32_t ipaddr, trg;

    myname = argv[0];
    target = argv[1];
    written = 0;
    n = readipaddr(&target, &ipaddr);
    if  (n >= 0 && n <= 32)
    {   lookfor = htonl(ipaddr);
        trg = masks[n];
    }else
    {   hp = gethostbyname(target);
        if (!hp) errx(EX_NOHOST, "cannot resolve %s: %s", target, hstrerror(h_errno));
        lookfor =   ((hp->h_addr[3] & 0xFF) << 24)
                  | ((hp->h_addr[2] & 0xFF) << 16)
                  | ((hp->h_addr[1] & 0xFF) << 8)
                  | (hp->h_addr[0] & 0xFF);
        trg = 0xFFFFFFFF;
    }
#ifdef DEBUG
    fprintf( stderr, "inet %u.%u.%u.%u mask %u.%u.%u.%u\n"
           , (lookfor) & 0xFF
           , (lookfor >> 8) & 0xFF
           , (lookfor >> 16) & 0xFF
           , (lookfor >> 24) & 0xFF
           , (trg) & 0xFF
           , (trg >> 8) & 0xFF
           , (trg >> 16) & 0xFF
           , (trg >> 24) & 0xFF
           );
#endif
    init_next();
    for (n = 0; get_next(fileno(stdin), &next) == sizeof(next); n++)
    {   if  (n == 0)
        {   if  (next.status & htonl(ARGUSCONTROL))
                bcopy(&next, &head, sizeof(next));
            else
            {   fprintf(stderr, "data error\n");                                      exit(EX_DATAERR);
            }
        }else if (next.status & htonl(IPPROTO))
        {
#ifdef DEBUG
            fprintf( stderr, "%5d src %08X dst %08X flt %08X msk %08X\n"
                   , n
                   , next.ws_ip_src.s_addr
                   , next.ws_ip_dst.s_addr
                   , lookfor
                   , trg
                   );
#endif
            if  (  (((next.ws_ip_src.s_addr ^ lookfor) & trg) == 0)
                || (((next.ws_ip_dst.s_addr ^ lookfor) & trg) == 0)
                )
            {   if  (!written)
                {   if  (sizeof(head) != write(fileno(stdout), &head, sizeof(head)))
                                                                  err( EX_IOERR, "can't write stdout");
                    written = 1;
                }
                if  (sizeof(next) != write(fileno(stdout), &next, sizeof(next)))
                                                                  err( EX_IOERR, "can't write stdout");
    }   }   }
    close(fileno(stdout));
    exit(EX_OK);
}
