/*-
 * Copyright (C) @BABOLO  2001 Jul 05
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * $Id: ra-ftreelter.c,v 1.2 2002/09/26 06:16:22 babolo Exp $ 
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sysexits.h>
#include <sys/uio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <err.h>


#define argtimeval timeval                       /* borrowed from argus (Carnegie Mellon University) */
#define arg_uint8   u_char
#define arg_int8    char
#define arg_uint16  u_short
#define arg_int16   short
#define arg_uint32  u_int
#define arg_int32   int                      /* END borrowed from argus (Carnegie Mellon University) */

#include <cons_def.h>
#include <cons_out.h>
#include "bufread.h"
#include "babololib.h"

/* typedef u_int32_t in_addr_t; For old FreeBSD */
#define FTREE_STRING "ftree"

char *myname;
u_int32_t masks[] =
{ 0x00000000
, 0x00000080, 0x000000C0, 0x000000E0, 0x000000F0
, 0x000000F8, 0x000000FC, 0x000000FE, 0x000000FF
, 0x000080FF, 0x0000C0FF, 0x0000E0FF, 0x0000F0FF
, 0x0000F8FF, 0x0000FCFF, 0x0000FEFF, 0x0000FFFF
, 0x0080FFFF, 0x00C0FFFF, 0x00E0FFFF, 0x00F0FFFF
, 0x00F8FFFF, 0x00FCFFFF, 0x00FEFFFF, 0x00FFFFFF
, 0x80FFFFFF, 0xC0FFFFFF, 0xE0FFFFFF, 0xF0FFFFFF
, 0xF8FFFFFF, 0xFCFFFFFF, 0xFEFFFFFF, 0xFFFFFFFF
};

int main(int argc, char *const *argv)
{   struct stat sb;
    struct ftree_node *ftree;
    const struct ftree_node *ftr0, *ftr1;
    int written, n, file;
    char *target, *p, *ftreen, ftree_dst;
    struct WriteStruct head, next;
    u_int32_t trgs, trgd;

    myname = argv[0];
    if  (argc < 2)                                                      errx(EX_USAGE, "No file name");
    ftreen = argv[1];
    if  (!ftreen)                                                       errx(EX_USAGE, "No file name");
    written = 0;
    if  ((file = open(ftreen, O_RDONLY)) < 0)                 err(EX_NOINPUT, "Can't open %s", ftreen);
    if  (fstat(file, &sb) < 0)                                err(EX_NOINPUT, "Can't stat %s", ftreen);
    ftree = mmap( NULL, sb.st_size, PROT_READ, 0, file, 0);
    if  ((void*)ftree == MAP_FAILED)                          err(EX_NOINPUT, "Can't mmap %s", ftreen);
    close(file);

    if  (argc < 3)                                                         errx(EX_USAGE, "No ftrees");
    target = argv[2];
    if  (strncmp(target, FTREE_STRING "s", strlen(FTREE_STRING "s")))
                                                                     errx(EX_USAGE, "Wait for ftrees");
    p = &target[strlen(FTREE_STRING "s")];
    trgs = (u_int32_t)strtol(p, &p, 10);
    if  (*p)                                                     errx(EX_USAGE, "Ftrees string error");

    if  (argc < 4)
        ftree_dst=0;
    else
    {   target = argv[3];
        if  (strncmp(target, FTREE_STRING "d", strlen(FTREE_STRING "d")))
                                                                     errx(EX_USAGE, "Wait for ftreed");
        p = &target[strlen(FTREE_STRING "d")];
        trgd = (u_int32_t)strtol(p, &p, 10);
        if  (*p)                                                 errx(EX_USAGE, "Ftreed string error");
        ftree_dst=1;
    }

    init_next();
    for (n = 0; get_next(fileno(stdin), &next) == sizeof(next); n++)
    {   if  (n == 0)
        {   if  (next.status & htonl(ARGUSCONTROL))
                bcopy(&next, &head, sizeof(next));
            else
            {   fprintf(stderr, "data error\n");                                      exit(EX_DATAERR);
            }
        }else if (next.status & htonl(IPPROTO))
        {   ftr0 = ftree_resolve(ftree, ntohl(next.ws_ip_src.s_addr));
            ftr1 = ftree_resolve(ftree, ntohl(next.ws_ip_dst.s_addr));
            if  ( ftree_dst
                ? ((ftr0->eq == trgs) && (ftr1->neq == trgd))
                : ((ftr0->eq == trgs) || (ftr1->eq == trgs))
                )
            {   if  (!written)
                {   if  (sizeof(head) != write(fileno(stdout), &head, sizeof(head)))
                                                                  err( EX_IOERR, "can't write stdout");
                    written = 1;
                }
                if  (sizeof(next) != write(fileno(stdout), &next, sizeof(next)))
                                                                  err( EX_IOERR, "can't write stdout");
    }   }   }
    close(fileno(stdout));
    exit(EX_OK);
}
