#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sysexits.h>
#include <sys/uio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <err.h>
#include "babololib.h"

struct tree_node
{   u_int32_t mask;
    u_int32_t value;
    u_int32_t eq;
    u_int32_t neq;
};

struct tree_node *ip_tree;

void print_ip(FILE *f, u_int32_t ip)
{   fprintf(f, "%3u", (ip >> 24) & 0xFF);
    fprintf(f, ".%u", (ip >> 16) & 0xFF);
    fprintf(f, ".%u", (ip >> 8) & 0xFF);
    fprintf(f, ".%u", ip & 0xFF);
}

void usage(int status)
{   fprintf( stderr
           , "traf-testpol @BABOLO V.M "VERS"  "DATE"\n"
             "   Usage:\n"
             "traf-testpol PolicyFile IPAddr\n"
           );
    exit(status);
}

int main(int argc, char **argv)
{   int ifile, p;
    struct stat sb;
    size_t len_tree;
    u_int32_t ipaddr;

    if (argc != 3) usage(EX_USAGE);
    if  ((ifile = open(argv[1], O_RDONLY)) < 0)              err(EX_NOINPUT, "Can't open %s", argv[1]);
    if  (fstat(ifile, &sb) < 0)                               err(EX_NOINPUT, "Can't stat input file");
    ip_tree = (struct tree_node*)mmap( NULL
                                     , sb.st_size
                                     , PROT_READ
                                     , 0
                                     , ifile
                                     , 0
                                     );
    if  ((void*)ip_tree == MAP_FAILED)                        err(EX_NOINPUT, "Can't mmap input file");
    len_tree = sb.st_size / sizeof(struct tree_node);
    close(ifile);
    p = readipaddr(&argv[2], &ipaddr);
    if  (p < 0 || p > 32)                                             errx(EX_USAGE, "IP input error");
    if  (p != 32)                                                errx(EX_USAGE, "Use /32 mask please");

    for (p = 0; ip_tree[p].mask != 0;)
    {   if  ((ip_tree[p].mask & ipaddr) == ip_tree[p].value)
        {   printf( "%5d == (m%08X v%08X i%08X)\n"
                  , p, ip_tree[p].mask & ipaddr, ip_tree[p].value, ipaddr
                  );
            p = ip_tree[p].eq;
        }else
        {   printf( "%5d <> (m%08X v%08X i%08X)\n"
                  , p, ip_tree[p].mask & ipaddr, ip_tree[p].value, ipaddr
                  );
            p = ip_tree[p].neq;
    }   }

    printf("%5d %5d %5d\n", p, ip_tree[p].eq, ip_tree[p].neq);
    exit(EX_OK);
}
